/*
   The object struct holds
    1) data for a list node
    2) a pointer to a function that can
       print the data in this object.
    3) a pointer to a function that can
       compare this object's data to another
       object's data
*/

//typedef void (*ToString)(void*);    // a function pointer type
//typedef int (*Equals)(void*,void*); // a function pointer type

struct Object {
   void *data;
   void (*toString)(void*);    // function pointer
   int (*equals)(void*,void*); // function pointer
 //ToString toString;          // another way to define the function pointer
 //Equals equals;              // another way to define the function pointer
};

// Here are some example functions that can be "stored" in a struct object.
void toStringInt(void* x) // x is really a int*
{
   printf( "%d", *((int*)x) );
}
void toStringDouble(void* x) // x is really a double*
{
   printf( "%f", *((double*)x) );
}
void toStringString(void* x) // x is really a char*
{
   printf( "%s", (char*)x );
}

int equalsInt(void *x, void *y) // x and y are really int*
{
   return *((int*)x) == *((int*)y);
}
int equalsDouble(void *x, void *y) // x and y are really double*
{
   return *((double*)x) == *((double*)y);
}
int equalsString(void *x, void *y) // x and y are really char*
{
   if (x == y) return 1;
   if (NULL == x || NULL == y) return 0;
   char *p1 = (char*)x, *p2 = (char*)y;
   while ('\0' != *p1 && '\0' != *p2) {
      if (*p1 != *p2) return 0;
      p1 += 1;
      p2 += 1;
   }
   if ('\0' != *p1 || '\0' != *p2) return 0;
   return 1;
}

// Here are three "constructors" for integer, double, and string objects.
struct Object *newInteger(int i)
{
   struct Object *obj = (struct Object*)malloc(sizeof(struct Object));

   // an "object" holds references to its "data" and its "methods"
   int *data = (int*)malloc(sizeof(int));
   *data = i;
   obj->data = data;
   obj->toString = toStringInt;
   obj->equals   = equalsInt;

   return obj;
}

struct Object *newDouble(double i)
{
   struct Object *obj = (struct Object*)malloc(sizeof(struct Object));

   // an "object" holds references to its "data" and its "methods"
   double *data = (double*)malloc(sizeof(double));
   *data = i;
   obj->data = data;
   obj->toString = toStringDouble;
   obj->equals   = equalsDouble;

   return obj;
}

struct Object *newString(char *i)
{
   struct Object *obj = (struct Object*)malloc(sizeof(struct Object));

   // an "object" holds references to its "data" and its "methods"
   obj->data = i;
   obj->toString = toStringString;
   obj->equals   = equalsString;

   return obj;
}


/*
EXERCISE: The Object class in Java has several important
methods defined in it like toString(), equals(), and clone().

https://docs.oracle.com/javase/8/docs/api/java/lang/Object.html

Above we imitated polymorphic toString() and equals() methods
for a "int class" a "double class" and a "string class". Give
each of these three "classes" a polymorphic clone() method.
Use those methods to give the list.h file a generic function
that can make a copy (clone) of a generic list.
*/
