/*
   This file declares a generic list with polymorphic functions.
*/
#ifndef _LIST_H_
#define _LIST_H_

#include <stdio.h>
#include <stdlib.h> // exit()
#include "object.h"

typedef struct list {
   struct Object *head;  // NULL value means empty list
   void *tail;
} List;

List* newList();  // construct an empty List
void freeList(List *list); // deallocate a List
int empty(List *list);
struct Object* head(List *list);
List* tail(List *list);
List* cons(struct Object *object, List *list);

int length(List *list);

// This is a "generic" function that determines if a value is in a list.
// Each "object" in a list is responsible for knowing how to compare itself.
int lookUp(void *data, List *list);

// This is a "generic" function that prints the data in a list. Each
// "object" in the list is responsible for knowing how to print itself.
void printList(List *list);

// This is a "generic" function that determines if two lists are equal.
// Each "object" in a list is responsible for knowing how to compare itself.
int equals(List *list1, List *list2);

#endif
