/*
   The object struct holds
    1) data for a list node
    2) a pointer to a function that can
       print the data in this object.
    3) a pointer to a function that can
       compare this object's data to another
       object's data
*/

#ifndef _OBJECT_H_
#define _OBJECT_H_

//typedef void (*ToString)(void*);    // a function pointer type
//typedef int (*Equals)(void*,void*); // a function pointer type

struct Object {
   void *data;
   void (*toString)(void*);    // function pointer
   int (*equals)(void*,void*); // function pointer
 //ToString toString;          // another way to define the function pointer
 //Equals equals;              // another way to define the function pointer
};

// Here are some example functions that can be "stored" in a struct object.
void toStringInt(void* x); // x is really a int*
void toStringDouble(void* x); // x is really a double*
void toStringString(void* x); // x is really a char*

int equalsInt(void *x, void *y); // x and y are really int*
int equalsDouble(void *x, void *y); // x and y are really double*
int equalsString(void *x, void *y); // x and y are really char*

// Here are three "constructors" for integer, double, and string objects.
struct Object *newInteger(int i);
struct Object *newDouble(double i);
struct Object *newString(char *i);

#endif

/*
EXERCISE: The Object class in Java has several important
methods defined in it like toString(), equals(), and clone().

https://docs.oracle.com/javase/8/docs/api/java/lang/Object.html

Above we imitated polymorphic toString() and equals() methods
for a "int class" a "double class" and a "string class". Give
each of these three "classes" a polymorphic clone() method.
Use those methods to give the list.h file a generic function
that can make a copy (clone) of a generic list.
*/
