/**
   This abstract generic class defines the interface for
   our List abstract data type (ADT).

   There will be two implementation of this interface,
   an "empty list" implementation, and a "non-empty list"
   implementation.

   In theory, we can characterize a List as follows,
      List<T> :: Empty
               | T : List<T>
   This is a recursive data type definition. In this
   implementation of the List ADT, the recursion in this
   definition is split over three files, the List<T> abstract
   class, the List_Nil<T> implementation, and the List_Node<T>
   implementation (which has an indirect recursive reference
   back to the List<T> class).
*/
public abstract class List<T>
{
   // The mutating part of the List API no longer works.
 //public abstract Object first();           // won't work
 //public abstract void rest();              // won't work
 //public abstract void insert(Object data); // won't work

   public abstract T head();
   public abstract List<T> tail();
   public abstract List<T> cons(T data);

   public abstract int length();
   public abstract boolean lookUp(T data);
   public abstract boolean empty();
   public abstract boolean equals(List<T> list);
   public abstract String toString();
   public abstract String toStringRec(); // Haskell's list notation
}

/*
   Notice a few things.
   1) This list ADT has a concrete notion of an empty list.
      It does not depend on nulls to denote empty. That is good.
   2) In fact, nulls cannot appear anywhere in these lists.
      That is enforced by the constructors, which is good.
   3) This list ADT is explicitly recursive, as it should be.
   4) But the recursive definition of the list ADT is split
      over three files. That's not good.
   5) The recursive definition of the length() method is
      split over two files. That's not good.
   6) The recursive definition of the lookUp() method is
      split over two files. That's not good.
   7) The List_Nil<T> class is needlessly generic since
      it doesn't hold any data. This adds complexity
      where it's not needed.
   8) It is possible to create subclasses of List_Nil<T>
      and List_Node<T> that break this abstract data type.
      But this is easy to fix, see the next design.

   The "algebraic data types" in Rust solve these problems.
*/
