/*
   This is the "non-empty list" implementation.

   This implementation is (indirectly) recursive, since a
   List_Node<T> extends List<T> and holds a reference to a List<T>.
*/
public class List_Node<T> extends List<T>
{
   T data;
   List<T> tail;  // recursive reference to List<T>

   public List_Node(T data) {
      this(data, new List_Nil<T>());
   }

   public List_Node(T data, List<T> tail) {
      if (null == data || null == tail) // tail is not allowed to be null
         throw new IllegalArgumentException();

      this.data = data;
      this.tail = tail;
   }


   public boolean empty() {
      return false;
   }


   public T head() {
      return data;
   }


   public List<T> tail() {
      return tail;
   }


   public List<T> cons(T data) {
      return new List_Node<T>(data, this);
   }


   public int length() {         // Notice how the structurally recursive
      return 1 + tail.length();  // length() method has its definition
   }                             // split over two files!


   public boolean lookUp(T data) {             // You also need to look at
      if (null == data)                        // the definition of lookUp()
         throw new IllegalArgumentException(); // in the List_Nil.java file.

      if (this.data.equals(data))  // non-empty list
         return true;
      else
         return this.tail.lookUp(data); // recursion
   }


   public boolean equals(List<T> list2) {
      if (this == list2)
         return true;
      else if (list2 instanceof List_Nil)
         return false;
      else if (! data.equals(list2.head()))  // its safe to call head()
         return false;
      else
         return tail.equals(list2.tail()); // recursion (its safe to call tail())
   }


   // Convert this list to Haskell's recursive notation for a list.
   public String toStringRec() {
      return data + " : " + tail.toStringRec(); // recursion
   }


   public String toString() {
      String result = "[" + data;
      for (List list = this.tail; ! (list instanceof List_Nil); list = list.tail()) {
         result += ", " + list.head();
      }
      result += "]";
      return result;
   }
}
